#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  simul.py
#
#  Copyright 2024 john <jcoppens@vostro.ampr.net>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.

# Cambios:
#
# 2024/03/13:
#   Por motivos de claridad:
#       - La carga del capacitor se guarda en la propiedad del capacitor en
#         Coulomb (no en voltaje como hicimos en clase)
#       - El valor de la resistencia y del capacitor se guarda en la
#         propiedad '.value'

import pylab as plt
from pdb import set_trace as st

dT = 0.005          # en segundos
Nsamples = 200      # Número de muestras

class Resistor:
    """ Simula el comportamento de una resistencia
    """
    def __init__(self, r_init):
        self.value = r_init

    def current(self, v):
        return v / self.value


class Capacitor:
    """ Simula el comportamento de un capacitor
    """
    def __init__(self, c_init, v_init = 0):
        """ El constructor recibe como parámetros:
                c_init      Capacitancia (en F)
                v_init      Tensión inicial del capacitor (en V, por defecto 0V)
        """
        self.value = c_init
        self.charge_to(v_init)

    def voltage(self):
        """ Devuelve la tensión actual del capacitor
        """
        return self.charge / self.value

    def discharge(self, i, dt):
        """ 'Descarga' al capacitor con una corriente <i> durante un período
            de <dt> segundos.
            Devuelve además la tensión final.
        """
        self.charge -= i*dt
        return self.voltage()

    def charge_to(self, v):
        """ Pone la tensión del capacitor inmediatamente a 'v' voltios,
            aunque el valor guardado se convierte a Coulomb
        """
        self.charge = v * self.value


class Inductor:
    """ Simula el comportamento de una inductancia
    """
    def __init__(self, l_init, i_init = 0):
        """ El constructor recibe como parámetros:
                l_init      Inductancia (en H)
                i_init      Corrient inicial del inductor (en A, por defecto 0)
                r_l         Resistence of inductor (en Ohms, por defecto 0.001)
        """
        self.value = l_init
        self.energy_to(i_init)
    def __init__(self, l_init, i_init = 0):
        """ El constructor recibe como parámetros:
                l_init      Inductancia (en H)
                i_init      Corrient inicial del inductor (en A, por defecto 0)
                r_l         Resistence of inductor (en Ohms, por defecto 0.001)
        """
        self.value = l_init
        self.energy_to(i_init)



def test_rc_discharge():
    """ Rutina diagnóstica para la librería. Conecta una resistencia de
        100kΩ en paralelo con un capacitor de 10µF, cargado inicialmente
        con 10V. (RC = 1 s)
    """
    V_INIT = 100.0

    r = Resistor(100000)
    print('La corriente en ', r.value, 'Ω, con 10V es', r.current(10), 'mA')

    c = Capacitor(10e-6, V_INIT)
    print('Carga inicial: ', c.voltage(), 'V (', c.charge, 'Coulomb)')

    print('Descarga del capacitor en la resistencia:')
    v = []  # lista de voltajes
    x = []  # lista de tiempos
    for t in range(Nsamples*2):
        v.append(c.voltage())
        x.append(dT*t)
        c.discharge(c.voltage()/r.value, dT)


    plt.plot(x, v)
    plt.grid(True)
    plt.ylabel('Tensión sobre el capacitor (V)')
    plt.xlabel('Tiempo (s)')
    plt.title('Descarga de un capacitor de 10µF con 100kΩ)')
    plt.show()


def main(args):
    test_rc_discharge()
    return 0


if __name__ == '__main__':
    import sys
    sys.exit(main(sys.argv))
