#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  simul.py
#
#  Copyright 2024 john <jcoppens@vostro.ampr.net>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.

# Cambios:
#
# 2024/03/13:
#   Por motivos de claridad:
#       - La carga del capacitor se guarda en la propiedad del capacitor en
#         Coulomb (no en voltaje como hicimos en clase)
#       - El valor de la resistencia y del capacitor se guarda en la
#         propiedad '.value'

import pylab as plt
from pdb import set_trace as st
from discrete import Resistor, Capacitor

dT = 0.005          # en segundos
Nsamples = 200      # Número de muestras

class Logic():
    def __init__(self, vcc = 5.0, v_thr_lo = 2.0, v_thr_hi = None):
        self.vcc = vcc
        self.v_thr_lo = v_thr_lo
        self.v_thr_hi = v_thr_hi
        self.v_mem = 0

    def level(self, v_in):
        if not self.v_thr_hi:
            return v_in >= self.v_thr_lo

        else:
            if v_in >= self.v_thr_hi:
                self.v_mem = self.vcc

            elif v_in < self.v_thr_lo:
                self.v_mem = 0

            return self.v_mem


class Buffer(Logic):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)


    def output(self, v_in):
        if self.level(v_in):
            return self.vcc
        else:
            return 0


class Inverter(Logic):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)


    def output(self, v_in):
        if not self.level(v_in):
            return self.vcc
        else:
            return 0


def plot_gate(gate, name):
    """ Grafica el comportamento de <gate> (que es una instancia de una
        compuerta (ej. Buffer, o Inverter)
        El gráfico varia la tensión en ascenso y luego en descenso.
    """
    x = []; y = []
    for i in range(Nsamples+1):
        v_in = i/Nsamples*gate.vcc
        # ~ print(v_in)
        x.append(v_in)
        y.append(gate.output(v_in))
    plt.plot(x, y, label = 'Ascendente')

    x = []; y = []
    for i in range(Nsamples, -1, -1):
        v_in = i/Nsamples*gate.vcc
        # ~ print(v_in)
        x.append(v_in)
        y.append(gate.output(v_in))
    plt.plot(x, y, label = 'Descendente')

    plt.legend()
    plt.title(name)
    plt.xlabel('Entrada')
    plt.ylabel('Salida')
    plt.grid(True)
    plt.show()


def plot_osc_1():
    gate = Inverter()
    v_in = 0
    x = []; y = []
    for i in range(Nsamples+1):
        v_in = gate.output(v_in)
        x.append(i)
        y.append(v_in)
    plt.plot(x, y, label = 'Oscilador')
    plt.title('Realimentación directa')
    plt.xlabel('Entrada')
    plt.ylabel('Salida')
    plt.grid(True)
    plt.show()


def plot_osc_2():
    gate = Inverter()
    c = Capacitor(1e-6)
    r = Resistor(1e5)
    x = []; y1 = []; y2 = []
    for i in range(Nsamples+1):
        # ~ st()
        v_out = gate.output(c.voltage())
        x.append(i)
        y1.append(c.voltage())
        y2.append(v_out)
        i_r = r.current(v_out - c.voltage())
        print(c.voltage(), v_out, i_r)
        c.discharge(-i_r, dT)

    plt.plot(x, y1, label = 'Capacitor')
    plt.plot(x, y2, label = 'Salida digital')
    plt.title('Realimentación con RC')
    plt.xlabel('Entrada')
    plt.ylabel('Salida')
    plt.grid(True)
    plt.show()


def plot_osc_555():
    gate = Inverter(vcc = 6, v_thr_lo = 2, v_thr_hi = 4)
    c = Capacitor(1e-6)
    r = Resistor(1e5)
    x = []; y1 = []; y2 = []
    for i in range(Nsamples+1):
        # ~ st()
        v_out = gate.output(c.voltage())
        x.append(i*dT)
        y1.append(c.voltage())
        y2.append(v_out)
        i_r = r.current(v_out - c.voltage())
        c.discharge(-i_r, dT)

    plt.plot(x, y1, label = 'Capacitor')
    plt.plot(x, y2, label = 'Salida digital')
    plt.title('Simulación de un NE555 (mas o menos)')
    plt.xlabel('Tiempo')
    plt.ylabel('Salida')
    plt.legend(loc = 'upper right')
    plt.grid(True)
    plt.show()



def test_logic():
    plot_gate(Buffer(), 'Buffer')                                   # 0318
    plot_gate(Inverter(), 'Inverter')                               # 0318
    plot_gate(Buffer(v_thr_hi = 3.2), 'Buffer con histeresis')      # 0319
    plot_gate(Inverter(v_thr_hi = 4), 'Inverter con histeresis')    # 0319
    plot_osc_1()                                                    # 0319
    plot_osc_2()                                                    # 0319
    plot_osc_3()                                                    # 0319
    plot_osc_555()                                                  # 0319


def main(args):
    # ~ test_rc_discharge()
    test_logic()
    return 0



if __name__ == '__main__':
    import sys
    sys.exit(main(sys.argv))
